-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

newTalent{
	name = "Enchant Lightning",
	type = {"technique/arcane-weapon", 1},
	mode = "sustained",
	points = 5,
	require = techs_req1,
	sustain_stamina = 20,
	no_energy = true,
	cooldown = 5,
	range = 10,
	tactical = { BUFF = 2 },
	direct_hit = true,
	reflectable = true,
	target = function(self, t)
		return {type="beam", range=self:getTalentRange(t), talent=t}
	end,
	getDamage = function(self, t) return self:combatTalentSpellDamage(t, 25, 300) end,
	do_trigger = function(self, t, target)
		if self.x == target.x and self.y == target.y then return nil end
		if rng.percent(20 + self:getTalentLevel(t) * (1 + self:getDex(9, true))) then
			local fatigue = (100 + 2 * self:combatFatigue()) / 100
			local mana_per_trigger = 6 * fatigue
			local mana = self:getMana() - 1
			--Mana cost per application
			if mana > mana_per_trigger then
				local l = self:lineFOV(target.x, target.y)
				l:set_corner_block()
				local lx, ly, is_corner_blocked = l:step(true)
				local x, y = lx, ly
				-- Check for terrain and friendly actors
				while lx and ly and not is_corner_blocked and core.fov.distance(self.x, self.y, lx, ly) <= 10 do
					local actor = game.level.map(lx, ly, engine.Map.ACTOR)
					if actor and (self:reactionToward(actor) >= 0) then
						break
					elseif game.level.map:checkEntity(lx, ly, engine.Map.TERRAIN, "block_move") then
						x, y = lx, ly
						break
					end
					x, y = lx, ly
					lx, ly = l:step(true)
				end
				print("[Enchant Lightning] autocast ")
				--Do the Lightning
				local tg = {type="beam", range=self:getTalentRange(t), talent=t}
				local dam = self:spellCrit(t.getDamage(self, t))
				self:project(tg, x, y, DamageType.LIGHTNING, rng.avg(dam / 3, dam, 3))
				local _ _, x, y = self:canProject(final_target, x, y)
				game.level.map:particleEmitter(self.x, self.y, math.max(math.abs(x-self.x), math.abs(y-self.y)), "lightning", {tx=x-self.x, ty=y-self.y})
				game:playSoundNear(self, "talents/lightning")
				self:incMana(-mana_per_trigger)
			end
		end
	end,
	activate = function(self, t)
		-- deactivate other talents and place on cooldown
		if self:isTalentActive(self.T_ENCHANT_FLAME) then
			self:useTalent(self.T_ENCHANT_FLAME)
		end
		if self:isTalentActive(self.T_ENCHANT_MANABLAST) then
			self:useTalent(self.T_ENCHANT_MANABLAST)
		end
		if self:isTalentActive(self.T_ENCHANT_FROST) then
			self:useTalent(self.T_ENCHANT_FROST)
		end
		return {}
	end,
	deactivate = function(self, t, p)
		return true
	end,
	info = function(self, t)
		local damage = t.getDamage(self,t)
		return ([[Allows one to use a melee weapon to focus spells, granting %d%% chance per melee attack to deliver a beam of Lightning as a free action at the target.
		The Lightning costs %0.2f mana each time it is triggered, and deals %0.2f to %0.2f damage.
		The damage will increase with your Spellpower and the chance increases with dexterity.]]):
		format(20 + self:getTalentLevel(t) * (1 + self:getDex(9, true)),(6 * (100 + 2 * self:combatFatigue()) / 100),damDesc(self, DamageType.LIGHTNING, damage / 3),
		damDesc(self, DamageType.LIGHTNING, damage))
	end,
}

newTalent{
	name = "Enchant Flame",
	type = {"technique/arcane-weapon", 2},
	mode = "sustained",
	points = 5,
	require = techs_req2,
	sustain_stamina = 20,
	no_energy = true,
	cooldown = 5,
	range = 10,
	tactical = { BUFF = 2 },
	direct_hit = true,
	reflectable = true,
	target = function(self, t)
		return {type="beam", range=self:getTalentRange(t), talent=t}
	end,
	getDamage = function(self, t) return self:combatTalentSpellDamage(t, 25, 250) end,
	do_trigger = function(self, t, target)
		if self.x == target.x and self.y == target.y then return nil end
		if rng.percent(20 + self:getTalentLevel(t) * (1 + self:getDex(9, true))) then
			local fatigue = (100 + 2 * self:combatFatigue()) / 100
			local mana_per_trigger = 10 * fatigue
			local mana = self:getMana() - 1
			--Mana cost per application
			if mana > mana_per_trigger then
				local l = self:lineFOV(target.x, target.y)
				l:set_corner_block()
				local lx, ly, is_corner_blocked = l:step(true)
				local x, y = lx, ly
				-- Check for terrain and friendly actors
				while lx and ly and not is_corner_blocked and core.fov.distance(self.x, self.y, lx, ly) <= 10 do
					local actor = game.level.map(lx, ly, engine.Map.ACTOR)
					if actor and (self:reactionToward(actor) >= 0) then
						break
					elseif game.level.map:checkEntity(lx, ly, engine.Map.TERRAIN, "block_move") then
						x, y = lx, ly
						break
					end
					x, y = lx, ly
					lx, ly = l:step(true)
				end
				--Do the Flame
				print("[Enchant Flame] autocast ")
				local grids = nil
				local tg = {type="beam", range=self:getTalentRange(t), talent=t}
				grids = self:project(tg, x, y, DamageType.FIREBURN, self:spellCrit(t.getDamage(self, t)))
				local _ _, x, y = self:canProject(tg, x, y)
				game.level.map:particleEmitter(self.x, self.y, tg.radius, "flamebeam", {tx=x-self.x, ty=y-self.y})
				if self:attr("burning_wake") and grids then
					for px, ys in pairs(grids) do
						for py, _ in pairs(ys) do
							game.level.map:addEffect(self, px, py, 4, engine.DamageType.INFERNO, self:attr("burning_wake"), 0, 5, nil, {type="inferno"}, nil, self:spellFriendlyFire())
						end
					end
				end
				game:playSoundNear(self, "talents/fire")
				self:incMana(-mana_per_trigger)
			end
		end
	end,
	activate = function(self, t)
		-- deactivate other talents and place on cooldown
		if self:isTalentActive(self.T_ENCHANT_LIGHTNING) then
			self:useTalent(self.T_ENCHANT_LIGHTNING)
		end
		if self:isTalentActive(self.T_ENCHANT_MANABLAST) then
			self:useTalent(self.T_ENCHANT_MANABLAST)
		end
		if self:isTalentActive(self.T_ENCHANT_FROST) then
			self:useTalent(self.T_ENCHANT_FROST)
		end
		return {}
	end,
	deactivate = function(self, t, p)
		return true
	end,
	info = function(self, t)
		local damage = t.getDamage(self,t)
		return ([[Allows one to use a melee weapon to focus spells, granting %d%% chance per melee attack to deliver a beam of Flame as a free action at the target.
		The Flame costs %0.2f mana each time it is triggered, and deals %0.2f damage over 3 turns.
		The damage will increase with your Spellpower and the chance increases with dexterity.]]):
		format(20 + self:getTalentLevel(t) * (1 + self:getDex(9, true)),(10 * (100 + 2 * self:combatFatigue()) / 100),damDesc(self, DamageType.FIRE,damage))
	end,
}

newTalent{
	name = "Enchant Frost",
	type = {"technique/arcane-weapon", 3},
	mode = "sustained",
	points = 5,
	require = techs_req3,
	sustain_stamina = 20,
	no_energy = true,
	cooldown = 5,
	range = 10,
	tactical = { BUFF = 2 },
	direct_hit = true,
	reflectable = true,
	target = function(self, t)
		return {type="beam", range=self:getTalentRange(t), talent=t}
	end,
	getDamage = function(self, t) return self:combatTalentSpellDamage(t, 25, 250) end,
	do_trigger = function(self, t, target)
		if self.x == target.x and self.y == target.y then return nil end
		if rng.percent(20 + self:getTalentLevel(t) * (1 + self:getDex(9, true))) then
			local fatigue = (100 + 2 * self:combatFatigue()) / 100
			local mana_per_trigger = 14 * fatigue
			local mana = self:getMana() - 1
			--Mana cost per application
			if mana > mana_per_trigger then
				local l = self:lineFOV(target.x, target.y)
				l:set_corner_block()
				local lx, ly, is_corner_blocked = l:step(true)
				local x, y = lx, ly
				-- Check for terrain and friendly actors
				while lx and ly and not is_corner_blocked and core.fov.distance(self.x, self.y, lx, ly) <= 10 do
					local actor = game.level.map(lx, ly, engine.Map.ACTOR)
					if actor and (self:reactionToward(actor) >= 0) then
						break
					elseif game.level.map:checkEntity(lx, ly, engine.Map.TERRAIN, "block_move") then
						x, y = lx, ly
						break
					end
					x, y = lx, ly
					lx, ly = l:step(true)
				end
				--Do the Frost
				print("[Enchant Frost] autocast ")
				local tg = {type="beam", range=self:getTalentRange(t), talent=t}
				self:project(tg, x, y, DamageType.ICE, self:spellCrit(t.getDamage(self, t)))
				local _ _, x, y = self:canProject(tg, x, y)
				game.level.map:particleEmitter(self.x, self.y, tg.radius, "icebeam", {tx=x-self.x, ty=y-self.y})
				game:playSoundNear(self, "talents/water")
				self:incMana(-mana_per_trigger)
			end
		end
	end,
	activate = function(self, t)
		-- deactivate other talents and place on cooldown
		if self:isTalentActive(self.T_ENCHANT_LIGHTNING) then
			self:useTalent(self.T_ENCHANT_LIGHTNING)
		end
		if self:isTalentActive(self.T_ENCHANT_MANABLAST) then
			self:useTalent(self.T_ENCHANT_MANABLAST)
		end
		if self:isTalentActive(self.T_ENCHANT_FLAME) then
			self:useTalent(self.T_ENCHANT_FLAME)
		end
		return {}
	end,
	deactivate = function(self, t, p)
		return true
	end,
	info = function(self, t)
		local damage = t.getDamage(self,t)
		return ([[Allows one to use a melee weapon to focus spells, granting %d%% chance per melee attack to deliver a beam of Frost as a free action at the target.
		The Frost costs %0.2f mana each time it is triggered, deals %0.2f damage, and may freeze targets in a block of ice for 2 turns.
		The damage will increase with your Spellpower and the chance increases with dexterity.]]):
		format(20 + self:getTalentLevel(t) * (1 + self:getDex(9, true)),(14 * (100 + 2 * self:combatFatigue()) / 100),damDesc(self, DamageType.COLD, damage))
	end,
}

newTalent{
	name = "Enchant Manablast",
	type = {"technique/arcane-weapon", 4},
	mode = "sustained",
	points = 5,
	require = techs_req4,
	sustain_stamina = 20,
	no_energy = true,
	cooldown = 5,
	range = 0,
	radius = function(self, t)
		return math.max(math.ceil(3 + self:getTalentLevel(t)),10)
	end,
	tactical = { BUFF = 2 },
	direct_hit = true,
	reflectable = true,
	target = function(self, t)
		return {type="cone", range=self:getTalentRange(t), talent=t}
	end,
	getDamage = function(self, t) return self:combatTalentSpellDamage(t, 25, 250) end,
	do_trigger = function(self, t, target)
		if self.x == target.x and self.y == target.y then return nil end
		if rng.percent(20 + self:getTalentLevel(t) * (1 + self:getDex(9, true))) then
			local fatigue = (100 + 2 * self:combatFatigue()) / 100
			local mana_per_trigger = 18 * fatigue
			local mana = self:getMana() - 1
			--Mana cost per application
			if mana > mana_per_trigger then
				--Do the Manablast
				print("[Enchant Manablast] autocast ")
				local tg = {type="cone", range=self:getTalentRange(t), radius=self:getTalentRadius(t), selffire=false, talent=t}
				self:project(tg, target.x, target.y, DamageType.ARCANE, self:spellCrit(t.getDamage(self, t)))
				local _ _, x, y = self:canProject(tg, target.x, target.y)
				game.level.map:particleEmitter(self.x, self.y, tg.radius, "breath_lightning", {radius=tg.radius, tx=target.x-self.x, ty=target.y-self.y})
				game:playSoundNear(self, "talents/arcane")
				self:incMana(-mana_per_trigger)
			end
		end
	end,
	activate = function(self, t)
		-- deactivate other talents and place on cooldown
		if self:isTalentActive(self.T_ENCHANT_LIGHTNING) then
			self:useTalent(self.T_ENCHANT_LIGHTNING)
		end
		if self:isTalentActive(self.T_ENCHANT_FLAME) then
			self:useTalent(self.T_ENCHANT_FLAME)
		end
		if self:isTalentActive(self.T_ENCHANT_FROST) then
			self:useTalent(self.T_ENCHANT_FROST)
		end
		return {}
	end,
	deactivate = function(self, t, p)
		return true
	end,
	info = function(self, t)
		local damage = t.getDamage(self,t)
		return ([[Allows one to use a melee weapon to focus spells, granting %d%% chance per melee attack to deliver a cone of Arcane power as a free action at the target.
		The Arcane power costs %0.2f mana each time it is triggered, deals %0.2f damage.
		The damage will increase with your Spellpower and the chance increases with dexterity.]]):
		format(20 + self:getTalentLevel(t) * (1 + self:getDex(9, true)),(18 * (100 + 2 * self:combatFatigue()) / 100), damDesc(self, DamageType.ARCANE, damage))
	end,
}
